// --------------------------------------------------------------------
// Scripted behaviour: Bouncing
//
// This file contains a simple script that causes the associated object
// to bounce around the canvas.
// --------------------------------------------------------------------
// Events: Bouncing.OnUpdate
// --------------------------------------------------------------------

// Constants
const float BOUNCE_SPEED = 150.0f;
const float BOUNCE_MARGIN = 0.0f;
const float BOUNCE_GRAVITY = 0.15f;

// Bouncing behaviour
event Bouncing.OnUpdate
{
	// Run this once
	if (sender.$init == false)
	{
		// Set initial velocity,rotation
		sender.$velocity = Vector2(randf()*6.0f - 3.0f,randf()*2.0f - 1.0f);	
		sender.$roll = sender.$velocity.AsFloat() * BOUNCE_SPEED * 0.02f;	
		sender.$init = true;
		
		// Adjust pivot point
		sender.pivot = sender.size * 0.5f;
		sender.position = sender.position + sender.size * 0.5f * sender.scale;		
	}

	// Get velocity, current position
	Vector2 velocity = sender.$velocity.AsVector2();
	Vector2 currPos = sender.position - (sender.pivot * sender.scale);

	// Apply gravity
	velocity.y += BOUNCE_GRAVITY * elapsed * BOUNCE_SPEED;

	// Check for horizontal bounce
	if ((velocity.x < 0 && currPos.x < 0.0f) || 
	(velocity.x > 0 && currPos.x + sender.size.x * sender.scale * (1.0f-BOUNCE_MARGIN) > 800)		)
	{
		velocity.x *= -1.0f;		// Invert horizontal velocity
		sender.$roll *= -1.0f;	// Flip rotation speed
	}

	// Check for vertical bounce
	if (velocity.y > 0 && currPos.y + sender.size.y * sender.scale * (1.0f-BOUNCE_MARGIN) > 600)
	{
		velocity.y *= -0.9f;		// Invert vertical velocity, lose energy
		if (abs(velocity.y) < 0.5f)	// If the bounce is small enough..
			velocity.y = 10.0f;		// ..Pop!
	}
	
	// Update new position, rotation and velocity
	sender.position = sender.position + velocity * elapsed * BOUNCE_SPEED;
	sender.rotation = sender.rotation + sender.$roll.AsFloat() * elapsed;
	sender.$velocity = velocity;
}