// Constants
const float THROW_BOUNCE_SPEED = 150.0f;
const float THROW_BOUNCE_MARGIN = 0.0f;
const float THROW_BOUNCE_GRAVITY = 0.15f;

// Called at the start of the page
void InitPickUpAndThrow(Object@ obj)
{
	// Set initial velocity,rotation
	obj.$velocity = Vector2(randf()*6.0f - 3.0f,randf()*2.0f - 1.0f);	
	
	// Adjust pivot point
	obj.pivot = obj.size * 0.5f;
	obj.position = obj.position + obj.size * 0.5f * obj.scale;
}


event PickUpAndThrow.OnMouseClick
{
	if (mouseButton == MB_LEFT)
	{
		// Begin dragging
		sender.$Drag = true;	
		sender.$OrigPos = sender.position;	
		sender.$MousePos = GetMousePosition();
	}
}

event PickUpAndThrow.OnMouseRelease
{
	if (mouseButton == MB_LEFT && sender.$Drag == true)
	{
		// Stop dragging
		sender.$Drag = false;
	}
}

event PickUpAndThrow.OnUpdate
{
	// Is dragging?
	if (sender.$Drag == true)
	{
		// Drag
		Vector2 DeltaMove = GetMousePosition() - sender.$MousePos.AsVector2();
		
		Vector2 PrevPos = sender.position;
		sender.position = sender.$OrigPos.AsVector2() + DeltaMove;
		
		sender.$velocity = (sender.position - PrevPos) * 0.5f + Vector2(-1.0f + randf()*2.0f, -1.0f + randf()*2.0f);
		sender.$resting = false;
	}
	else
	{
		// Bounce around
		if (_VAR_EXISTS(sender,"velocity") == false)
			InitPickUpAndThrow(sender);
		
		// Get velocity, current position
		Vector2 velocity = sender.$velocity.AsVector2();
		Vector2 currPos = sender.position - (sender.pivot * sender.scale);

		// Apply gravity
		velocity.y += THROW_BOUNCE_GRAVITY * elapsed * THROW_BOUNCE_SPEED;

		// Check for horizontal bounce
		if ((velocity.x < 0 && currPos.x < 0.0f) || 
		(velocity.x > 0 && currPos.x + sender.size.x * sender.scale * (1.0f-THROW_BOUNCE_MARGIN) > 800)		)
		{
			velocity.x *= -1.0f;		// Invert horizontal velocity
		}

		// Check for vertical bounce
		if (velocity.y > 0 && currPos.y + sender.size.y * sender.scale * (1.0f-THROW_BOUNCE_MARGIN) > 600)
		{
			sender.position.y -= 1.0f;
			velocity.y *= -0.8f;		// Invert vertical velocity, lose energy
			if (abs(velocity.y) < 0.5f)	// If the bounce is small enough..
			{
				if (abs(velocity.x) > 2.0f)
				{
					// steal some from horizontal
					velocity.y += 2.0f;
					velocity.x -= (velocity.x > 0) ? 0.5f : -0.5f;
				}
				else
					sender.$resting = true;
			}
							
			velocity.x *= 0.75f;
			if (abs(velocity.x) < 0.01f)
				velocity.x = 0.0f;
		}
		
		// Update new position, rotation and velocity
		if (sender.$resting == false)
		{
			sender.position = sender.position + velocity * elapsed * THROW_BOUNCE_SPEED;
			sender.$velocity = velocity;		
			
		}
	}
	
}