// Art asset
const string PARTICLE_ASSET = "flare";

// Target layer
const string LAYER_NAME = "Sparkle";

// Misc. Settings
const uint PARTICLE_COUNT = 50;
const float SPAWN_TIME = 0.01f;
const float PARTICLE_TIME = 1.0f;
const float PARTICLE_SCALE = 0.5f;

// Global storage
array<Object@> g_ParticlePool;
Vector2 g_OldMousePos;

// At start of page...
event ParticleMaker.OnCreate
{
	Print("Creating particles... ("+PARTICLE_COUNT+")");

	// Find layer
	SceneLayer@ TargetLayer = Scene.GetLayer(LAYER_NAME);

	// Create particles
	g_ParticlePool.Resize(PARTICLE_COUNT);
	for(uint i = 0; i < PARTICLE_COUNT; i++)
	{
		// Create object in layer (or default)
		Object@ newObject;
		if (TargetLayer !is null)
			@newObject = Scene.CreateObjectByName(PARTICLE_ASSET, TargetLayer);
		else
			@newObject = Scene.CreateObjectByName(PARTICLE_ASSET);
		
		if (newObject !is null)
		{
			newObject.blending = BLEND_ADD;
			newObject.pivot = newObject.size / 2;
			newObject.visible = false;
			newObject.rotation = randf()*TWOPI;
			
			newObject.$active = false;
			newObject.OnUpdate = "Particle";
		}
		@g_ParticlePool[i] = newObject;
	}
	
	// Initialize global variables
	$time = SPAWN_TIME;
	$index = 0;
}

// Every frame...
event ParticleMaker.OnUpdate
{
	// Has mouse moved?
	Vector2 MousePos = GetMousePosition();
	
	if (MousePos != g_OldMousePos)
	{
		// Wait for particle
		$time -= elapsed;
		if ($time.AsFloat() <= 0.0f)
		{
			$time += SPAWN_TIME;
			
			// Create particle (from pool)
			CreateParticle($index, g_OldMousePos - MousePos);

			// Increment particle index
			$index += 1;
			if ($index == PARTICLE_COUNT)
				$index = 0;
		}
		// Store mouse position
		g_OldMousePos = MousePos;
	}
}

// Function that initializes one particle
void CreateParticle(uint index, Vector2 Velocity)
{
	if (index >= PARTICLE_COUNT)
		return;
		
	// Get object from particle pool
	Object@ p = g_ParticlePool[index];
	
	// Initialize local variables
	p.$active = true;
	p.$counter = 1.0f + randf()*0.5f;
	p.$velocity = Velocity.Normalized() + Vector2(randf()*2.0f-1.0f,randf()*2.0f-1.0f);

	// Set properties
	p.visible = true;
	p.position = GetMousePosition() + Vector2(randf()*2.0f-1.0f,randf()*2.0f-1.0f) * 8.0f;
	p.scale = PARTICLE_SCALE;
	
}

// Every frame..
event Particle.OnUpdate
{
	// Don't run if inactive
	if (sender.$active == false)
		return;
		
	// Count down
	sender.$counter -= elapsed / PARTICLE_TIME;
	float value = sender.$counter;
	
	// Change scale/rotation/opacity
	sender.scale = value * PARTICLE_SCALE;
	sender.rotation = sender.rotation + elapsed * TWOPI;
	sender.opacity = clamp(value,0.0f,1.0f);

	// Move
	sender.position.Add(sender.$velocity.AsVector2() * elapsed * 50.0f / PARTICLE_TIME);

	// At end of life?
	if (value <= 0.0f)
	{
		// Disable/hide
		sender.$active = false;
		sender.visible = false;
	}
}

